@@ -39,11 +39,13 @@ programmatically.  A document can have multiple indexes by adding extra
 
 '''
 import pymongo
-
+from collections import defaultdict
 from mongoalchemy.util import classproperty
 from mongoalchemy.query_expression import QueryField
 from mongoalchemy.fields import ObjectIdField, Field, BadValueException, SCALAR_MODIFIERS
-from mongoalchemy.exceptions import DocumentException, MissingValueException, ExtraValueException, FieldNotRetrieved
+from mongoalchemy.exceptions import DocumentException, MissingValueException, ExtraValueException, FieldNotRetrieved, BadFieldSpecification
+
+document_type_registry = defaultdict(dict)
 
 class DocumentMeta(type):
     def __new__(mcs, classname, bases, class_dict):
@@ -69,6 +71,14 @@ class DocumentMeta(type):
             if not isinstance(field, QueryField):
                 continue
             new_class._fields[name] = field.get_type()
+        
+        # 3. register type
+        if new_class.config_namespace != None:
+            name = new_class.config_full_name
+            if name == None:
+                name = new_class.__name__
+            document_type_registry[new_class.config_namespace][name] = new_class
+        
         return new_class
 
 class Document(object):
@@ -80,19 +90,37 @@ class Document(object):
         This field can be overridden in subclasses if the default ID is not
         acceptable '''
     
+    config_namespace = 'global'
+    ''' The namespace is used to determine how string class names should be 
+        looked up.  If an instance of DocumentField is created using a string,
+        it will be looked up using the value of this variable and the string.
+        To have more than one namespace create a subclass of Document
+        overriding this class variable.  To turn off caching all together, 
+        create a subclass where namespace is set to None.  Doing this will 
+        disable using strings to look up document names, which will make 
+        creating self-referencing documents impossible.  The default value is
+        "global"
+    '''
+    
+    config_full_name = None
+    ''' If namespaces are being used, the key for a class is normally
+        the class name.  In some cases the same class name may be used in 
+        different modules.  This field allows a longer unambiguous name
+        to be given.  It may also be used in error messages or string 
+        representations of the class
+    '''
+    
     config_extra_fields = 'error'
     ''' Controls the method to use when dealing with fields passed in to the
         document constructor.  Possible values are 'error' and 'ignore'. Any 
         fields which couldn't be mapped can be retrieved (and edited) using
         :func:`~Document.get_extra_fields` '''
     
-    def __init__(self, retrieved_fields=None, **kwargs):
-        '''
-        **Parameters**:
-            * retrieved_fields: The names of the fields returned when loading \
+    def __init__(self, retrieved_fields=None, loading_from_db=False, **kwargs):
+        ''' :param retrieved_fields: The names of the fields returned when loading \
                 a partial object.  This argument should not be explicitly set \
                 by subclasses
-            * \*\*kwargs:  The values for all of the fields in the document. \
+            :param \*\*kwargs:  The values for all of the fields in the document. \
                 Any additional fields will raise a :class:`~mongoalchemy.document.ExtraValueException` and \ 
                 any missing (but required) fields will raise a :class:`~mongoalchemy.document.MissingValueException`. \
                 Both types of exceptions are subclasses of :class:`~mongoalchemy.document.DocumentException`.
@@ -113,7 +141,7 @@ class Document(object):
                 continue
             
             if name in kwargs:
-                setattr(self, name, kwargs[name])
+                getattr(cls, name).set_value(self, kwargs[name], from_db=loading_from_db)
                 continue
         
         for k in kwargs:
@@ -123,15 +151,23 @@ class Document(object):
                 else:
                     raise ExtraValueException(k)
     
-    def get_dirty_ops(self):
+    def get_dirty_ops(self, with_required=False):
         ''' Returns a dict with the update operations necessary to make the 
             changes to this object to the database version.  It is mainly used
             internally for :func:`~mongoalchemy.session.Session.update` but
             may be useful for diagnostic purposes as well.
+            
+            :param with_required: Also include any field which is required.  This \
+                is useful if the method is being called for the purposes of \
+                an upsert where all required fields must always be sent.
         '''
         update_expression = {}
         for name, field in self.get_fields().iteritems():
             dirty_ops = field.dirty_ops(self)
+            if not dirty_ops and with_required and field.required:
+                dirty_ops = field.update_ops(self)
+                if not dirty_ops:
+                    raise MissingValueException(name)
             
             for op, values in dirty_ops.iteritems():
                 update_expression.setdefault(op, {})
@@ -147,14 +183,14 @@ class Document(object):
     
     @classmethod
     def get_fields(cls):
-        '''Returns a dict mapping the names of the fields in a document 
+        ''' Returns a dict mapping the names of the fields in a document 
             or subclass to the associated :class:`~mongoalchemy.fields.Field`
         '''
         return cls._fields
     
     @classmethod
     def class_name(cls):
-        '''Returns the name of the class. The name of the class is also the 
+        ''' Returns the name of the class. The name of the class is also the 
             default collection name.  
             
             .. seealso:: :func:`~Document.get_collection_name`
@@ -205,8 +241,7 @@ class Document(object):
         ''' Save this object to the database and set the ``_id`` field of this
             document to the returned id.
             
-            **Parameters**:
-                * db: The pymongo database to write to
+            :param db: The pymongo database to write to
         '''
         collection = db[self.get_collection_name()]
         for index in self.get_indexes():
@@ -248,13 +283,12 @@ class Document(object):
     
     @classmethod
     def unwrap(cls, obj, fields=None):
-        '''Returns an instance of this document class based on the mongo object 
+        ''' Returns an instance of this document class based on the mongo object 
             ``obj``.  This is done by using the ``unwrap()`` methods of the 
             underlying fields to set values.
             
-            **Parameters**:
-                * obj: a ``SON`` object returned from a mongo database
-                * fields: A list of :class:`mongoalchemy.query.QueryField` objects \
+            :param obj: a ``SON`` object returned from a mongo database
+            :param fields: A list of :class:`mongoalchemy.query.QueryField` objects \
                     for the fields to load.  If ``None`` is passed all fields  \
                     are loaded
             '''
@@ -281,7 +315,13 @@ class Document(object):
         
         if fields != None:
             params['retrieved_fields'] = fields
-        return cls(**params)
+        obj = cls(loading_from_db=True, **params)
+        obj.__mark_clean()
+        return obj
+    
+    def __mark_clean(self):
+        self._dirty.clear()
+        
 
 class DictDoc(object):
     ''' Adds a mapping interface to a document.  Supports ``__getitem__`` and 
@@ -308,7 +348,7 @@ class DictDoc(object):
         return self[name]
     
     def __contains__(self, name):
-        '''Return whether a field is present.  Fails if ``name`` is not a 
+        ''' Return whether a field is present.  Fails if ``name`` is not a 
             field or ``name`` is not set on the document or if ``name`` was 
             not a field retrieved from the database
         '''
@@ -330,8 +370,18 @@ class DocumentField(Field):
     
     def __init__(self, document_class, **kwargs):
         super(DocumentField, self).__init__(**kwargs)
-        self.type = document_class
-        self.document_class = document_class
+        self.__type = document_class
+    
+    @property
+    def type(self):
+        if not isinstance(self.__type, basestring) and issubclass(self.__type, Document):
+            return self.__type
+        if self.parent.config_namespace == None:
+            raise BadFieldSpecification('Document namespace is None.  Strings are not allowed for DocumentFields')
+        type = document_type_registry[self.parent.config_namespace].get(self.__type)
+        if type == None or not issubclass(type, Document):
+            raise BadFieldSpecification('No type found for %s.  Maybe it has not been imported yet and is not registered?' % self.__type)
+        return type
     
     def dirty_ops(self, instance):
         ''' Returns a dict of the operations needed to update this object.  
@@ -356,7 +406,7 @@ class DocumentField(Field):
     def subfields(self):
         ''' Returns the fields that can be retrieved from the enclosed 
             document.  This function is mainly used internally'''
-        return self.document_class.get_fields()
+        return self.type.get_fields()
     
     def sub_type(self):
         return self.type
@@ -365,11 +415,10 @@ class DocumentField(Field):
         ''' Called before wrapping.  Calls :func:`~DocumentField.is_valid_unwrap` and 
             raises a :class:`BadValueException` if validation fails            
             
-            **Parameters**: 
-                * value: The value to validate
-                * fields: The fields being returned if this is a partial \
-                    document. They will be ignored when validating the fields \
-                    of ``value``
+            :param value: The value to validate
+            :param fields: The fields being returned if this is a partial \
+                document. They will be ignored when validating the fields \
+                of ``value``
         '''
         try:
             self.validate_unwrap(value, fields=fields)
@@ -378,19 +427,19 @@ class DocumentField(Field):
         return True
     
     def wrap(self, value):
-        '''Validate ``value`` and then use the document's class to wrap the 
+        ''' Validate ``value`` and then use the document's class to wrap the 
             value'''
         self.validate_wrap(value)
         return self.type.wrap(value)
     
     def unwrap(self, value, fields=None):
-        '''Validate ``value`` and then use the document's class to unwrap the 
+        ''' Validate ``value`` and then use the document's class to unwrap the 
             value'''
         self.validate_unwrap(value, fields=fields)
         return self.type.unwrap(value, fields=fields)
     
     def validate_wrap(self, value):
-        ''' Checks that ``value`` is an instance of ``DocumentField.document_class``.
+        ''' Checks that ``value`` is an instance of ``DocumentField.type``.
             if it is, then validation on its fields has already been done and
             no further validation is needed.
         '''
@@ -398,7 +447,7 @@ class DocumentField(Field):
             self._fail_validation_type(value, self.type)
     
     def validate_unwrap(self, value, fields=None):
-        '''Validates every field in the underlying document type.  If ``fields`` 
+        ''' Validates every field in the underlying document type.  If ``fields`` 
             is not ``None``, only the fields in ``fields`` will be checked.
         '''
         try:
@@ -410,7 +459,7 @@ class BadIndexException(Exception):
     pass
 
 class Index(object):
-    '''This class is  used in the class definition of a :class:`~Document` to 
+    ''' This class is  used in the class definition of a :class:`~Document` to 
         specify a single, possibly compound, index.  ``pymongo``'s ``ensure_index``
         will be called on each index before a database operation is executed 
         on the owner document class.
@@ -434,28 +483,25 @@ class Index(object):
         self.__drop_dups = False
     
     def ascending(self, name):
-        '''Add a descending index for ``name`` to this index.
+        ''' Add a descending index for ``name`` to this index.
         
-        **Parameters**:
-            * name: Name to be used in the index
+            :param name: Name to be used in the index
         '''
         self.components.append((name, Index.ASCENDING))
         return self
 
     def descending(self, name):
-        '''Add a descending index for ``name`` to this index.
-        
-        **Parameters**:
-            * name: Name to be used in the index
+        ''' Add a descending index for ``name`` to this index.
+            
+            :param name: Name to be used in the index
         '''
         self.components.append((name, Index.DESCENDING))
         return self
     
     def unique(self, drop_dups=False):
-        '''Make this index unique, optionally dropping duplicate entries.
+        ''' Make this index unique, optionally dropping duplicate entries.
                 
-        **Parameters**:
-            * drop_dups: Drop duplicate objects while creating the unique \
+            :param drop_dups: Drop duplicate objects while creating the unique \
                 index?  Default to ``False``
         '''
         self.__unique = True
@@ -465,8 +511,7 @@ class Index(object):
     def ensure(self, collection):
         ''' Call the pymongo method ``ensure_index`` on the passed collection.
             
-            **Parameters**:
-                * collection: the ``pymongo`` collection to ensure this index \
+            :param collection: the ``pymongo`` collection to ensure this index \
                     is on
         '''
         collection.ensure_index(self.components, unique=self.__unique, 
