@@ -1,6 +1,6 @@
 from nose.tools import *
 from mongoalchemy.session import Session
-from mongoalchemy.document import Document, Index, DocumentField, MissingValueException, DocumentException, DictDoc
+from mongoalchemy.document import Document, Index, DocumentField, MissingValueException, DocumentException, DictDoc, document_type_registry
 from mongoalchemy.fields import *
 from test.util import known_failure
 
@@ -37,6 +37,9 @@ class DocA(Document):
 
 # Tests
 
+def test_setup():
+    document_type_registry.clear()
+
 def get_session():
     return Session.connect('unit-testing')
 
@@ -221,6 +224,112 @@ def is_valid_unwrap_test_false():
 def wrong_unwrap_type_test():
     DocA.unwrap({ 'test_doc2' : { 'int1' : 1 } })
 
+@raises(MissingValueException)
+def test_upsert_with_required():
+    class D(Document):
+        a = IntField()
+        c = IntField()
+        b = IntField(required=False)
+    
+    s = get_session()
+    s.clear_collection(D)
+    s.update(D(b=4, c=4), id_expression=D.b == 4, upsert=True)
+
+def test_upsert_with_no_changes():
+    class D(Document):
+        a = IntField()
+        c = IntField()
+        b = IntField(required=False)
+    
+    s = get_session()
+    s.clear_collection(D)
+    s.update(D(a=1, b=4, c=4), id_expression=D.b == 4, upsert=True)
+    d = s.query(D).one()
+    s.update(d, upsert=True)
+
+def test_unwrapped_is_not_dirty():
+    class D(Document):
+        a = IntField()
+    s = get_session()
+    s.clear_collection(D)
+    s.insert(D(a=1))
+    d = s.query(D).one()
+    assert len(d.get_dirty_ops()) == 0, len(d.get_dirty_ops())
+
+def test_update_with_unset():
+    class D(Document, DictDoc):
+        a = IntField()
+        c = IntField()
+        b = IntField(required=False)
+    
+    s = get_session()
+    s.clear_collection(D)
+    d = D(a=1, b=4, c=4)
+    s.update(d, id_expression=D.b == 4, upsert=True)
+    d = s.query(D).one()
+    del d.c
+    s.update(d)
+    d = s.query(D).one()
+    assert 'c' not in d
+
+@with_setup(test_setup)
+def test_self_reference():
+    class D(Document):
+        d = DocumentField('D', required=False)
+        a = IntField()
+    d = D(d=D(a=5), a=4)
+    assert d.wrap() == { 'd' : { 'a' : 5 }, 'a' : 4 }
+    s = get_session()
+    s.clear_collection(D)
+    s.insert(d)
+    d_from_db = s.query(D).one()
+    assert d_from_db.d.a == d.d.a
+
+@with_setup(test_setup)
+def test_config_full_name():
+    class E(Document):
+        d = DocumentField('ma.D')
+    class D(Document):
+        config_full_name = 'ma.D'
+        a = IntField()
+    e = E(d=D(a=4))
+    assert e.d.a == 4
+
+@with_setup(test_setup)
+def test_config_in_list():
+    class E(Document):
+        d = ListField(ListField(DocumentField('ma.D')))
+    class D(Document):
+        config_full_name = 'ma.D'
+        a = IntField()
+    e = E(d=[[D(a=4)]])
+    assert e.wrap() == { 'd' : [[{'a':4}]] }
+
+@with_setup(test_setup)
+@raises(BadFieldSpecification)
+def test_bad_string_doc():
+    class D(Document):
+        e = DocumentField('E')
+    D(e=5).wrap()
+
+@with_setup(test_setup)
+@raises(BadFieldSpecification)
+def test_namespaces_disabled():
+    class D(Document):
+        config_namespace = None
+        e = DocumentField('E')
+    D(e=5).wrap()
+
+@with_setup(test_setup)
+def test_set_parent_on_subtypes():
+    class D(Document):
+        a = IntField()
+    
+    class ParentDoc(Document):
+        t = TupleField(DocumentField('D'))
+        e = EnumField(DocumentField('D'), D(a=1))
+        d = DictField(DocumentField('D'))
+
 # test DictDoc
 
 def test_dictdoc_contains():
